# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ...utils.filemanip import split_filename
from ..base import (traits, TraitedSpec, File, StdOutCommandLine,
                    StdOutCommandLineInputSpec)


class QBallMXInputSpec(StdOutCommandLineInputSpec):
    basistype = traits.Enum(
        'rbf',
        'sh',
        argstr='-basistype %s',
        desc=('Basis function type. "rbf" to use radial basis functions '
              '"sh" to use spherical harmonics'),
        usedefault=True)
    scheme_file = File(
        exists=True,
        argstr='-schemefile %s',
        mandatory=True,
        desc='Specifies the scheme file for the diffusion MRI data')
    order = traits.Int(
        argstr='-order %d',
        units='NA',
        desc=(
            'Specific to sh. Maximum order of the spherical harmonic series. '
            'Default is 4.'))
    rbfpointset = traits.Int(
        argstr='-rbfpointset %d',
        units='NA',
        desc=(
            'Specific to rbf. Sets the number of radial basis functions to use. '
            'The value specified must be present in the Pointsets directory. '
            'The default value is 246.'))
    rbfsigma = traits.Float(
        argstr='-rbfsigma %f',
        units='NA',
        desc=
        ('Specific to rbf. Sets the width of the interpolating basis functions. '
         'The default value is 0.2618 (15 degrees).'))
    smoothingsigma = traits.Float(
        argstr='-smoothingsigma %f',
        units='NA',
        desc=(
            'Specific to rbf. Sets the width of the smoothing basis functions. '
            'The default value is 0.1309 (7.5 degrees).'))


class QBallMXOutputSpec(TraitedSpec):
    qmat = File(exists=True, desc='Q-Ball reconstruction matrix')


class QBallMX(StdOutCommandLine):
    """
    Generates a reconstruction matrix for Q-Ball. Used in LinRecon with
    the same scheme file to reconstruct data.

    Example 1
    ---------
    To create  a linear transform matrix using Spherical Harmonics (sh).

    >>> import nipype.interfaces.camino as cam
    >>> qballmx = cam.QBallMX()
    >>> qballmx.inputs.scheme_file = 'A.scheme'
    >>> qballmx.inputs.basistype = 'sh'
    >>> qballmx.inputs.order = 6
    >>> qballmx.run()            # doctest: +SKIP

    Example 2
    ---------
    To create  a linear transform matrix using Radial Basis Functions
    (rbf). This command uses the default setting of rbf sigma = 0.2618
    (15 degrees), data smoothing sigma = 0.1309 (7.5 degrees), rbf
    pointset 246

    >>> import nipype.interfaces.camino as cam
    >>> qballmx = cam.QBallMX()
    >>> qballmx.inputs.scheme_file = 'A.scheme'
    >>> qballmx.run()              # doctest: +SKIP

    The linear transform matrix from any of these two examples can then
    be run over each voxel using LinRecon

    >>> qballcoeffs = cam.LinRecon()
    >>> qballcoeffs.inputs.in_file = 'SubjectA.Bfloat'
    >>> qballcoeffs.inputs.scheme_file = 'A.scheme'
    >>> qballcoeffs.inputs.qball_mat = 'A_qmat.Bdouble'
    >>> qballcoeffs.inputs.normalize = True
    >>> qballcoeffs.inputs.bgmask = 'brain_mask.nii'
    >>> qballcoeffs.run()             # doctest: +SKIP
    """
    _cmd = 'qballmx'
    input_spec = QBallMXInputSpec
    output_spec = QBallMXOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['qmat'] = os.path.abspath(self._gen_outfilename())
        return outputs

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.scheme_file)
        return name + '_qmat.Bdouble'


class LinReconInputSpec(StdOutCommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=1,
        desc='voxel-order data filename')
    scheme_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=2,
        desc='Specifies the scheme file for the diffusion MRI data')
    qball_mat = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=3,
        desc='Linear transformation matrix.')
    normalize = traits.Bool(
        argstr='-normalize',
        desc=('Normalize the measurements and discard '
              'the zero measurements before the linear transform.'))
    log = traits.Bool(
        argstr='-log',
        desc=('Transform the log measurements rather than the '
              'measurements themselves'))
    bgmask = File(exists=True, argstr='-bgmask %s', desc='background mask')


class LinReconOutputSpec(TraitedSpec):
    recon_data = File(exists=True, desc='Transformed data')


class LinRecon(StdOutCommandLine):
    """
    Runs a linear transformation in each voxel.

    Reads  a  linear  transformation from the matrix file assuming the
    imaging scheme specified in the scheme file. Performs the linear
    transformation on the data in every voxel and outputs the result to
    the standard output. The ouput in every voxel is actually: ::

        [exit code, ln(S(0)), p1, ..., pR]

    where p1, ..., pR are the parameters of the reconstruction.
    Possible exit codes are:

        - 0. No problems.
        - 6. Bad data replaced by substitution of zero.

    The matrix must be R by N+M where N+M is the number of measurements
    and R is the number of parameters of the reconstruction. The matrix
    file contains binary double-precision floats. The matrix elements
    are stored row by row.

    Example
    ---------
    First run QBallMX and create a linear transform matrix using
    Spherical Harmonics (sh).

    >>> import nipype.interfaces.camino as cam
    >>> qballmx = cam.QBallMX()
    >>> qballmx.inputs.scheme_file = 'A.scheme'
    >>> qballmx.inputs.basistype = 'sh'
    >>> qballmx.inputs.order = 4
    >>> qballmx.run()            # doctest: +SKIP

    Then run it over each voxel using LinRecon

    >>> qballcoeffs = cam.LinRecon()
    >>> qballcoeffs.inputs.in_file = 'SubjectA.Bfloat'
    >>> qballcoeffs.inputs.scheme_file = 'A.scheme'
    >>> qballcoeffs.inputs.qball_mat = 'A_qmat.Bdouble'
    >>> qballcoeffs.inputs.normalize = True
    >>> qballcoeffs.run()         # doctest: +SKIP
    """
    _cmd = 'linrecon'
    input_spec = LinReconInputSpec
    output_spec = LinReconOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['recon_data'] = os.path.abspath(self._gen_outfilename())
        return outputs

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.scheme_file)
        return name + '_recondata.Bdouble'


class MESDInputSpec(StdOutCommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='-inputfile %s',
        mandatory=True,
        position=1,
        desc='voxel-order data filename')
    inverter = traits.Enum(
        'SPIKE',
        'PAS',
        argstr='-filter %s',
        position=2,
        mandatory=True,
        desc=
        ('The inversion index specifies the type of inversion to perform on the data.'
         'The currently available choices are:'
         'Inverter name  | Inverter parameters'
         '---------------|------------------'
         'SPIKE          | bd (b-value x diffusivity along the fibre.)'
         'PAS            | r'))
    inverter_param = traits.Float(
        argstr='%f',
        units='NA',
        position=3,
        mandatory=True,
        desc=
        ('Parameter associated with the inverter. Cf. inverter description for'
         'more information.'))
    fastmesd = traits.Bool(
        argstr='-fastmesd',
        requires=['mepointset'],
        desc=
        ('Turns off numerical integration checks and fixes the integration point set size at that of'
         'the index specified by -basepointset..'))
    mepointset = traits.Int(
        argstr='-mepointset %d',
        units='NA',
        desc=
        ('Use a set of directions other than those in the scheme file for the deconvolution kernel.'
         'The number refers to the number of directions on the unit sphere. For example, '
         '"-mepointset 54" uses the directions in "camino/PointSets/Elec054.txt".'
         ))
    scheme_file = File(
        exists=True,
        argstr='-schemefile %s',
        mandatory=True,
        desc='Specifies the scheme file for the diffusion MRI data')
    bgmask = File(exists=True, argstr='-bgmask %s', desc='background mask')
    inputdatatype = traits.Enum(
        'float',
        'char',
        'short',
        'int',
        'long',
        'double',
        argstr='-inputdatatype %s',
        desc=
        ('Specifies the data type of the input file: "char", "short", "int", "long",'
         '"float" or "double". The input file must have BIG-ENDIAN ordering.'
         'By default, the input type is "float".'))


class MESDOutputSpec(TraitedSpec):
    mesd_data = File(exists=True, desc='MESD data')


class MESD(StdOutCommandLine):
    """
    MESD is a general program for maximum entropy spherical deconvolution.
    It also runs PASMRI, which is a special case of spherical deconvolution.
    The input data must be in voxel order.

    The format of the output in each voxel is:
    { exitcode, ln(A^star(0)), lambda_0, lambda_1, ..., lambda_N }

    The  exitcode  contains  the  results of three tests. The first test thresholds
    the maximum relative error between the numerical integrals computed at con-
    vergence and those computed using a larger test point set; if the error is
    greater than a threshold the exitcode is increased from zero to one as a
    warning; if it is greater than a larger threshold the exitcode is increased to
    two to suggest failure. The  second  test  thresholds  the predicted  error in
    numerical integrals computed using the test point set; if the predicted error
    is greater than a threshold the exitcode is increased by 10. The third test
    thresholds the RMS error between the measurements and their predictions from
    the fitted deconvolution; if the errors are greater than a threshold, the exit
    code is increased by 100. An exitcode of 112 means that all three tests were
    failed and the result is likely to be unreliable.  If all is well the exitcode
    is zero. Results are often still reliable even if one or two of the tests are
    failed.

    Other possible exitcodes are:

        - 5 - The optimization failed to converge
        - -1 - Background
        - -100 - Something wrong in the MRI data, e.g. negative or zero measurements,
          so that the optimization could not run.

    The  standard  MESD  implementation  is computationally demanding, particularly
    as the number of measurements increases (computation is approximately O(N^2),
    where N is the number of measurements). There are two ways to obtain significant
    computational speed-up:

    i) Turn off error checks and use a small point set for computing numerical
    integrals in the algorithm by adding the flag -fastmesd. Sakaie CDMRI 2008
    shows that using the smallest point set  (-basepointset  0)  with  no
    error  checks  usually  has only a minor effect on the output of the algorithm,
    but provides a major reduction in computation time. You can increase the point
    set size using -basepointset with an argument higher than 0, which may produce
    better results in some voxels, but will increase computation time, which
    approximately doubles every time the point set index increases by 1.

    ii) Reduce the complexity of the maximum entropy encoding using -mepointset <X>.
    By default <X> = N, the number of measurements, and is the number of parameters
    in the max.  ent. representation of the  output  function, ie  the  number of
    lambda parameters, as described in Jansons and Alexander Inverse Problems 2003.
    However, we can represent the function using less components and <X> here
    specifies the number of lambda parameters. To obtain speed-up, set <X>
    < N; complexity become O(<X>^2) rather than O(N^2). Note that <X> must be chosen
    so that the camino/PointSets directory contains a point set  with  that  number
    of  elements.  When  -mepointset decreases, the  numerical  integration checks
    make less and less of a difference and smaller point sets for numerical
    integration (see -basepointset) become adequate. So when <X> is low -fastmesd is
    worth using to get even more speed-up.

    The choice of <X> is a parameter of the technique. Too low and you lose angular
    resoloution; too high and you see no computational benefit and may even suffer
    from overfitting. Empirically, we  have  found  that  <X>=16 often  gives  good
    results and good speed up, but it is worth trying a few values a comparing
    performance. The reduced encoding is described in the following ISMRM abstract:
    Sweet and Alexander "Reduced Encoding Persistent Angular Structure" 572 ISMRM 2010.

    Example
    ---------
    Run MESD on every voxel of the data file SubjectA.Bfloat using the PASMRI kernel.

    >>> import nipype.interfaces.camino as cam
    >>> mesd = cam.MESD()
    >>> mesd.inputs.in_file = 'SubjectA.Bfloat'
    >>> mesd.inputs.scheme_file = 'A.scheme'
    >>> mesd.inputs.inverter = 'PAS'
    >>> mesd.inputs.inverter_param = 1.4
    >>> mesd.run()            # doctest: +SKIP
    """
    _cmd = 'mesd'
    input_spec = MESDInputSpec
    output_spec = MESDOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['mesd_data'] = os.path.abspath(self._gen_outfilename())
        return outputs

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.scheme_file)
        return name + '_MESD.Bdouble'


class SFPeaksInputSpec(StdOutCommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='-inputfile %s',
        mandatory=True,
        desc='Voxel-order data of spherical functions')
    inputmodel = traits.Enum(
        'sh',
        'maxent',
        'rbf',
        argstr='-inputmodel %s',
        mandatory=True,
        desc=
        ('Type of functions input via in_file. Currently supported options are: '
         '  sh - Spherical harmonic series. Specify the maximum order of the SH series '
         '       with the "order" attribute if different from the default of 4. '
         '  maxent - Maximum entropy representations output by MESD. The reconstruction '
         '           directions input to MESD must be specified. By default this is the '
         '           same set of gradient directions (excluding zero gradients) in the '
         '           scheme file, so specify the "schemefile" attribute unless the '
         '           "mepointset" attribute was set in MESD. '
         '  rbf - Sums of radial basis functions. Specify the pointset with the attribute '
         '        "rbfpointset" if different from the default. See QBallMX.'))
    order = traits.Int(
        argstr='-order %d',
        units='NA',
        desc='Specific to sh. Maximum order of the spherical harmonic series.')
    scheme_file = File(
        exists=True,
        argstr='%s',
        desc='Specific to maxent. Specifies the scheme file.')
    rbfpointset = traits.Int(
        argstr='-rbfpointset %d',
        units='NA',
        desc=(
            'Specific to rbf. Sets the number of radial basis functions to use. '
            'The value specified must be present in the Pointsets directory. '
            'The default value is 246.'))
    mepointset = traits.Int(
        argstr='-mepointset %d',
        units='NA',
        desc=
        ('Use a set of directions other than those in the scheme file for the deconvolution '
         'kernel. The number refers to the number of directions on the unit sphere. '
         'For example, "mepointset = 54" uses the directions in "camino/PointSets/Elec054.txt" '
         'Use this option only if you told MESD to use a custom set of directions with the same '
         'option. Otherwise, specify the scheme file with the "schemefile" attribute.'
         ))
    numpds = traits.Int(
        argstr='-numpds %d',
        units='NA',
        desc='The largest number of peak directions to output in each voxel.')
    noconsistencycheck = traits.Bool(
        argstr='-noconsistencycheck',
        desc=
        'Turns off the consistency check. The output shows all consistencies as true.'
    )
    searchradius = traits.Float(
        argstr='-searchradius %f',
        units='NA',
        desc=
        'The search radius in the peak finding algorithm. The default is 0.4 (cf. "density")'
    )
    density = traits.Int(
        argstr='-density %d',
        units='NA',
        desc=
        ('The  number  of  randomly  rotated icosahedra to use in constructing the set of points for '
         'random sampling in the peak finding algorithm. Default is 1000, which works well for very '
         'spiky maxent functions. For other types of function, it is reasonable to set the density '
         'much lower and increase the search radius slightly, which speeds up the computation.'
         ))
    pointset = traits.Int(
        argstr='-pointset %d',
        units='NA',
        desc=
        ('To sample using an evenly distributed set of points instead. The integer can be '
         '0, 1, ..., 7. Index 0 gives 1082 points, 1 gives 1922, 2 gives 3002, 3 gives 4322, '
         '4 gives 5882, 5 gives 8672, 6 gives 12002, 7 gives 15872.'))
    pdthresh = traits.Float(
        argstr='-pdthresh %f',
        units='NA',
        desc=
        ('Base threshold on the actual peak direction strength divided by the mean of the '
         'function.  The default is 1.0 (the peak must be equal or greater than the mean).'
         ))
    stdsfrommean = traits.Float(
        argstr='-stdsfrommean %f',
        units='NA',
        desc=
        ('This is the number of standard deviations of the function to be added to the '
         '"pdthresh" attribute in the peak directions pruning.'))


class SFPeaksOutputSpec(TraitedSpec):
    peaks = File(exists=True, desc='Peaks of the spherical functions.')


class SFPeaks(StdOutCommandLine):
    """
    Finds the peaks of spherical functions.

    This utility reads coefficients of the spherical functions and
    outputs a list of peak directions of the function. It computes the
    value of the function at each of a set of sample points. Then it
    finds local maxima by finding all points at which the function is
    larger than for any other point within a fixed search radius (the
    default  is 0.4). The utility then uses Powell's algorithm to
    optimize the position of each local maximum. Finally the utility
    removes duplicates and tiny peaks with function value smaller than
    some threshold, which is the mean of the function plus some number
    of standard deviations. By default the program checks for con-
    sistency with a second set of starting points, but skips the
    optimization step. To speed up execution, you can turn off the con-
    sistency check by setting the noconsistencycheck flag to True.

    By  default, the utility constructs a set of sample points by
    randomly rotating a unit icosahedron repeatedly (the default is 1000
    times, which produces a set of 6000 points) and concatenating the
    lists of vertices. The 'pointset = <index>' attribute can tell the
    utility to use an evenly distributed set of points (index 0 gives
    1082 points, 1 gives 1922, 2 gives 4322, 3 gives 8672, 4 gives 15872,
    5 gives 32762, 6 gives 72032), which is quicker, because you can get
    away with fewer points. We estimate that you can use a factor of 2.5
    less evenly distributed points than randomly distributed points and
    still expect similar performance levels.

    The output for each voxel is:

    - exitcode (inherited from the input data).
    - ln(A(0))
    - number of peaks found.
    - flag for consistency with a repeated run (number of directions is
      the same and the directions are the same to within a threshold.)
    - mean(f).
    - std(f).
    - direction 1 (x, y, z, f, H00, H01, H10, H11).
    - direction 2 (x, y, z, f, H00, H01, H10, H11).
    - direction 3 (x, y, z, f, H00, H01, H10, H11).

    H is the Hessian of f at the peak. It is the matrix: ::

        [d^2f/ds^2 d^2f/dsdt]
        [d^2f/dtds d^2f/dt^2]
        = [H00 H01]
          [H10 H11]

    where s and t are orthogonal coordinates local to the peak.

    By default the maximum number of peak directions output in each
    voxel is three. If less than three directions are found, zeros are
    output for later directions. The peaks are ordered by the value of
    the function at the peak. If more than the maximum number of
    directions are found only the strongest ones are output. The maximum
    number can be changed setting the 'numpds' attribute.

    The utility can read various kinds of spherical function, but must
    be told what kind of function is input using the 'inputmodel'
    attribute. The description of the 'inputmodel' attribute lists
    additional information required by SFPeaks for each input model.


    Example
    ---------
    First run QBallMX and create a linear transform matrix using
    Spherical Harmonics (sh).

    >>> import nipype.interfaces.camino as cam
    >>> sf_peaks = cam.SFPeaks()
    >>> sf_peaks.inputs.in_file = 'A_recon_params.Bdouble'
    >>> sf_peaks.inputs.inputmodel = 'sh'
    >>> sf_peaks.inputs.order = 4
    >>> sf_peaks.inputs.density = 100
    >>> sf_peaks.inputs.searchradius = 1.0
    >>> sf_peaks.run()          # doctest: +SKIP
    """
    _cmd = 'sfpeaks'
    input_spec = SFPeaksInputSpec
    output_spec = SFPeaksOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['peaks'] = os.path.abspath(self._gen_outfilename())
        return outputs

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '_peaks.Bdouble'



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..legacy import antsIntroduction


def test_antsIntroduction_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bias_field_correction=dict(argstr='-n 1', ),
        dimension=dict(
            argstr='-d %d',
            position=1,
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        force_proceed=dict(argstr='-f 1', ),
        input_image=dict(
            argstr='-i %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        inverse_warp_template_labels=dict(argstr='-l', ),
        max_iterations=dict(
            argstr='-m %s',
            sep='x',
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_prefix=dict(
            argstr='-o %s',
            usedefault=True,
        ),
        quality_check=dict(argstr='-q 1', ),
        reference_image=dict(
            argstr='-r %s',
            copyfile=True,
            extensions=None,
            mandatory=True,
        ),
        similarity_metric=dict(argstr='-s %s', ),
        transformation_model=dict(
            argstr='-t %s',
            usedefault=True,
        ),
    )
    inputs = antsIntroduction.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_antsIntroduction_outputs():
    output_map = dict(
        affine_transformation=dict(extensions=None, ),
        input_file=dict(extensions=None, ),
        inverse_warp_field=dict(extensions=None, ),
        output_file=dict(extensions=None, ),
        warp_field=dict(extensions=None, ),
    )
    outputs = antsIntroduction.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



"""The dtitk module provides classes for interfacing with the `Diffusion
Tensor Imaging Toolkit (DTI-TK)
<http://dti-tk.sourceforge.net/pmwiki/pmwiki.php>`_ command line tools.

Top-level namespace for dti-tk.
"""

# from .base import ()
from .registration import (Rigid, Affine, Diffeo,
                           ComposeXfm, DiffeoSymTensor3DVol, AffSymTensor3DVol,
                           AffScalarVol, DiffeoScalarVol)
from .utils import (TVAdjustVoxSp, SVAdjustVoxSp, TVResample, SVResample,
                    TVtool, BinThresh)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
import os.path as op

from ..base import (CommandLineInputSpec, traits, TraitedSpec, File, isdefined)
from .base import MRTrix3Base


class BuildConnectomeInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='input tractography')
    in_parc = File(
        exists=True, argstr='%s', position=-2, desc='parcellation file')
    out_file = File(
        'connectome.csv',
        argstr='%s',
        mandatory=True,
        position=-1,
        usedefault=True,
        desc='output file after processing')

    nthreads = traits.Int(
        argstr='-nthreads %d',
        desc='number of threads. if zero, the number'
        ' of available cpus will be used',
        nohash=True)

    vox_lookup = traits.Bool(
        argstr='-assignment_voxel_lookup',
        desc='use a simple voxel lookup value at each streamline endpoint')
    search_radius = traits.Float(
        argstr='-assignment_radial_search %f',
        desc='perform a radial search from each streamline endpoint to locate '
        'the nearest node. Argument is the maximum radius in mm; if no node is'
        ' found within this radius, the streamline endpoint is not assigned to'
        ' any node.')
    search_reverse = traits.Float(
        argstr='-assignment_reverse_search %f',
        desc='traverse from each streamline endpoint inwards along the '
        'streamline, in search of the last node traversed by the streamline. '
        'Argument is the maximum traversal length in mm (set to 0 to allow '
        'search to continue to the streamline midpoint).')
    search_forward = traits.Float(
        argstr='-assignment_forward_search %f',
        desc='project the streamline forwards from the endpoint in search of a'
        'parcellation node voxel. Argument is the maximum traversal length in '
        'mm.')

    metric = traits.Enum(
        'count',
        'meanlength',
        'invlength',
        'invnodevolume',
        'mean_scalar',
        'invlength_invnodevolume',
        argstr='-metric %s',
        desc='specify the edge'
        ' weight metric')

    in_scalar = File(
        exists=True,
        argstr='-image %s',
        desc='provide the associated image '
        'for the mean_scalar metric')

    in_weights = File(
        exists=True,
        argstr='-tck_weights_in %s',
        desc='specify a text scalar '
        'file containing the streamline weights')

    keep_unassigned = traits.Bool(
        argstr='-keep_unassigned',
        desc='By default, the program discards the'
        ' information regarding those streamlines that are not successfully '
        'assigned to a node pair. Set this option to keep these values (will '
        'be the first row/column in the output matrix)')
    zero_diagonal = traits.Bool(
        argstr='-zero_diagonal',
        desc='set all diagonal entries in the matrix '
        'to zero (these represent streamlines that connect to the same node at'
        ' both ends)')


class BuildConnectomeOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output response file')


class BuildConnectome(MRTrix3Base):
    """
    Generate a connectome matrix from a streamlines file and a node
    parcellation image

    Example
    -------

    >>> import nipype.interfaces.mrtrix3 as mrt
    >>> mat = mrt.BuildConnectome()
    >>> mat.inputs.in_file = 'tracks.tck'
    >>> mat.inputs.in_parc = 'aparc+aseg.nii'
    >>> mat.cmdline                               # doctest: +ELLIPSIS
    'tck2connectome tracks.tck aparc+aseg.nii connectome.csv'
    >>> mat.run()                                 # doctest: +SKIP
    """

    _cmd = 'tck2connectome'
    input_spec = BuildConnectomeInputSpec
    output_spec = BuildConnectomeOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = op.abspath(self.inputs.out_file)
        return outputs


class LabelConfigInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='input anatomical image')
    in_config = File(
        exists=True,
        argstr='%s',
        position=-2,
        desc='connectome configuration file')
    out_file = File(
        'parcellation.mif',
        argstr='%s',
        mandatory=True,
        position=-1,
        usedefault=True,
        desc='output file after processing')

    lut_basic = File(
        argstr='-lut_basic %s',
        desc='get information from '
        'a basic lookup table consisting of index / name pairs')
    lut_fs = File(
        argstr='-lut_freesurfer %s',
        desc='get information from '
        'a FreeSurfer lookup table(typically "FreeSurferColorLUT'
        '.txt")')
    lut_aal = File(
        argstr='-lut_aal %s',
        desc='get information from the AAL '
        'lookup table (typically "ROI_MNI_V4.txt")')
    lut_itksnap = File(
        argstr='-lut_itksnap %s',
        desc='get information from an'
        ' ITK - SNAP lookup table(this includes the IIT atlas '
        'file "LUT_GM.txt")')
    spine = File(
        argstr='-spine %s',
        desc='provide a manually-defined '
        'segmentation of the base of the spine where the streamlines'
        ' terminate, so that this can become a node in the connection'
        ' matrix.')
    nthreads = traits.Int(
        argstr='-nthreads %d',
        desc='number of threads. if zero, the number'
        ' of available cpus will be used',
        nohash=True)


class LabelConfigOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output response file')


class LabelConfig(MRTrix3Base):
    """
    Re-configure parcellation to be incrementally defined.

    Example
    -------

    >>> import nipype.interfaces.mrtrix3 as mrt
    >>> labels = mrt.LabelConfig()
    >>> labels.inputs.in_file = 'aparc+aseg.nii'
    >>> labels.inputs.in_config = 'mrtrix3_labelconfig.txt'
    >>> labels.cmdline                               # doctest: +ELLIPSIS
    'labelconfig aparc+aseg.nii mrtrix3_labelconfig.txt parcellation.mif'
    >>> labels.run()                                 # doctest: +SKIP
    """

    _cmd = 'labelconfig'
    input_spec = LabelConfigInputSpec
    output_spec = LabelConfigOutputSpec

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []

        if not isdefined(self.inputs.in_config):
            from distutils.spawn import find_executable
            path = find_executable(self._cmd)
            if path is None:
                path = os.getenv(MRTRIX3_HOME, '/opt/mrtrix3')
            else:
                path = op.dirname(op.dirname(path))

            self.inputs.in_config = op.join(
                path, 'src/dwi/tractography/connectomics/'
                'example_configs/fs_default.txt')

        return super(LabelConfig, self)._parse_inputs(skip=skip)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = op.abspath(self.inputs.out_file)
        return outputs


class LabelConvertInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-4,
        desc='input anatomical image')
    in_lut = File(
        exists=True,
        argstr='%s',
        mandatory=True,
        position=-3,
        desc='get information from '
        'a basic lookup table consisting of index / name pairs')
    in_config = File(
        exists=True,
        argstr='%s',
        position=-2,
        desc='connectome configuration file')
    out_file = File(
        'parcellation.mif',
        argstr='%s',
        mandatory=True,
        position=-1,
        usedefault=True,
        desc='output file after processing')
    spine = File(
        argstr='-spine %s',
        desc='provide a manually-defined '
        'segmentation of the base of the spine where the streamlines'
        ' terminate, so that this can become a node in the connection'
        ' matrix.')
    num_threads = traits.Int(
        argstr='-nthreads %d',
        desc='number of threads. if zero, the number'
        ' of available cpus will be used',
        nohash=True)


class LabelConvertOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='the output response file')


class LabelConvert(MRTrix3Base):
    """
    Re-configure parcellation to be incrementally defined.

    Example
    -------

    >>> import nipype.interfaces.mrtrix3 as mrt
    >>> labels = mrt.LabelConvert()
    >>> labels.inputs.in_file = 'aparc+aseg.nii'
    >>> labels.inputs.in_config = 'mrtrix3_labelconfig.txt'
    >>> labels.inputs.in_lut = 'FreeSurferColorLUT.txt'
    >>> labels.cmdline
    'labelconvert aparc+aseg.nii FreeSurferColorLUT.txt mrtrix3_labelconfig.txt parcellation.mif'
    >>> labels.run()                                 # doctest: +SKIP
    """

    _cmd = 'labelconvert'
    input_spec = LabelConvertInputSpec
    output_spec = LabelConvertOutputSpec

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []

        if not isdefined(self.inputs.in_config):
            from nipype.utils.filemanip import which
            path = which(self._cmd)
            if path is None:
                path = os.getenv(MRTRIX3_HOME, '/opt/mrtrix3')
            else:
                path = op.dirname(op.dirname(path))

            self.inputs.in_config = op.join(
                path, 'src/dwi/tractography/connectomics/'
                'example_configs/fs_default.txt')

        return super(LabelConvert, self)._parse_inputs(skip=skip)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = op.abspath(self.inputs.out_file)
        return outputs



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ..base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                    TraitedSpec, File, Directory, traits, isdefined,
                    InputMultiPath, OutputMultiPath)


class GenerateCsfClippedFromClassifiedImageInputSpec(CommandLineInputSpec):
    inputCassifiedVolume = File(
        desc="Required: input tissue label image",
        exists=True,
        argstr="--inputCassifiedVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s")


class GenerateCsfClippedFromClassifiedImageOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class GenerateCsfClippedFromClassifiedImage(SEMLikeCommandLine):
    """title: GenerateCsfClippedFromClassifiedImage

category: FeatureCreator

description: Get the distance from a voxel to the nearest voxel of a given tissue type.

version: 0.1.0.$Revision: 1 $(alpha)

documentation-url: http:://www.na-mic.org/

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: This tool was written by Hans J. Johnson.

"""

    input_spec = GenerateCsfClippedFromClassifiedImageInputSpec
    output_spec = GenerateCsfClippedFromClassifiedImageOutputSpec
    _cmd = " GenerateCsfClippedFromClassifiedImage "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}
    _redirect_x = False



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import pytest

from ....testing import example_data
from ...niftyreg import get_custom_path
from ...niftyreg.tests.test_regutils import no_nifty_tool
from .. import (UnaryMaths, BinaryMaths, BinaryMathsInteger, TupleMaths, Merge)


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_maths'), reason="niftyseg is not installed")
def test_unary_maths():

    # Create a node object
    unarym = UnaryMaths()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_maths', env_dir='NIFTYSEGDIR')
    assert unarym.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        unarym.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    unarym.inputs.in_file = in_file
    unarym.inputs.operation = 'otsu'
    unarym.inputs.output_datatype = 'float'

    expected_cmd = '{cmd} {in_file} -otsu -odt float {out_file}'.format(
        cmd=cmd, in_file=in_file, out_file='im1_otsu.nii')

    assert unarym.cmdline == expected_cmd


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_maths'), reason="niftyseg is not installed")
def test_binary_maths():

    # Create a node object
    binarym = BinaryMaths()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_maths', env_dir='NIFTYSEGDIR')
    assert binarym.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        binarym.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    binarym.inputs.in_file = in_file
    binarym.inputs.operand_value = 2.0
    binarym.inputs.operation = 'sub'
    binarym.inputs.output_datatype = 'float'

    cmd_tmp = '{cmd} {in_file} -sub 2.00000000 -odt float {out_file}'
    expected_cmd = cmd_tmp.format(
        cmd=cmd, in_file=in_file, out_file='im1_sub.nii')

    assert binarym.cmdline == expected_cmd


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_maths'), reason="niftyseg is not installed")
def test_int_binary_maths():

    # Create a node object
    ibinarym = BinaryMathsInteger()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_maths', env_dir='NIFTYSEGDIR')
    assert ibinarym.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        ibinarym.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    ibinarym.inputs.in_file = in_file
    ibinarym.inputs.operand_value = 2
    ibinarym.inputs.operation = 'dil'
    ibinarym.inputs.output_datatype = 'float'

    expected_cmd = '{cmd} {in_file} -dil 2 -odt float {out_file}'.format(
        cmd=cmd, in_file=in_file, out_file='im1_dil.nii')

    assert ibinarym.cmdline == expected_cmd


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_maths'), reason="niftyseg is not installed")
def test_tuple_maths():

    # Create a node object
    tuplem = TupleMaths()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_maths', env_dir='NIFTYSEGDIR')
    assert tuplem.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        tuplem.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    op_file = example_data('im2.nii')
    tuplem.inputs.in_file = in_file
    tuplem.inputs.operation = 'lncc'
    tuplem.inputs.operand_file1 = op_file
    tuplem.inputs.operand_value2 = 2.0
    tuplem.inputs.output_datatype = 'float'

    cmd_tmp = '{cmd} {in_file} -lncc {op} 2.00000000 -odt float {out_file}'
    expected_cmd = cmd_tmp.format(
        cmd=cmd, in_file=in_file, op=op_file, out_file='im1_lncc.nii')

    assert tuplem.cmdline == expected_cmd


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_maths'), reason="niftyseg is not installed")
def test_merge():

    # Create a node object
    merge = Merge()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_maths', env_dir='NIFTYSEGDIR')
    assert merge.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        merge.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    file1 = example_data('im2.nii')
    file2 = example_data('im3.nii')
    merge.inputs.in_file = in_file
    merge.inputs.merge_files = [file1, file2]
    merge.inputs.dimension = 2
    merge.inputs.output_datatype = 'float'

    cmd_tmp = '{cmd} {in_file} -merge 2 2 {f1} {f2} -odt float {out_file}'
    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        f1=file1,
        f2=file2,
        out_file='im1_merged.nii')

    assert merge.cmdline == expected_cmd



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Average


def test_Average_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        avgdim=dict(argstr='-avgdim %s', ),
        binarize=dict(argstr='-binarize', ),
        binrange=dict(argstr='-binrange %s %s', ),
        binvalue=dict(argstr='-binvalue %s', ),
        check_dimensions=dict(
            argstr='-check_dimensions',
            xor=('check_dimensions', 'no_check_dimensions'),
        ),
        clobber=dict(
            argstr='-clobber',
            usedefault=True,
        ),
        copy_header=dict(
            argstr='-copy_header',
            xor=('copy_header', 'no_copy_header'),
        ),
        debug=dict(argstr='-debug', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filelist=dict(
            argstr='-filelist %s',
            extensions=None,
            mandatory=True,
            xor=('input_files', 'filelist'),
        ),
        format_byte=dict(
            argstr='-byte',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_double=dict(
            argstr='-double',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_filetype=dict(
            argstr='-filetype',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_float=dict(
            argstr='-float',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_int=dict(
            argstr='-int',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_long=dict(
            argstr='-long',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_short=dict(
            argstr='-short',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_signed=dict(
            argstr='-signed',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        format_unsigned=dict(
            argstr='-unsigned',
            xor=('format_filetype', 'format_byte', 'format_short',
                 'format_int', 'format_long', 'format_float', 'format_double',
                 'format_signed', 'format_unsigned'),
        ),
        input_files=dict(
            argstr='%s',
            mandatory=True,
            position=-2,
            sep=' ',
            xor=('input_files', 'filelist'),
        ),
        max_buffer_size_in_kb=dict(
            argstr='-max_buffer_size_in_kb %d',
            usedefault=True,
        ),
        no_check_dimensions=dict(
            argstr='-nocheck_dimensions',
            xor=('check_dimensions', 'no_check_dimensions'),
        ),
        no_copy_header=dict(
            argstr='-nocopy_header',
            xor=('copy_header', 'no_copy_header'),
        ),
        nonormalize=dict(
            argstr='-nonormalize',
            xor=('normalize', 'nonormalize'),
        ),
        normalize=dict(
            argstr='-normalize',
            xor=('normalize', 'nonormalize'),
        ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=['input_files'],
            name_template='%s_averaged.mnc',
            position=-1,
        ),
        quiet=dict(
            argstr='-quiet',
            xor=('verbose', 'quiet'),
        ),
        sdfile=dict(
            argstr='-sdfile %s',
            extensions=None,
        ),
        two=dict(argstr='-2', ),
        verbose=dict(
            argstr='-verbose',
            xor=('verbose', 'quiet'),
        ),
        voxel_range=dict(argstr='-range %d %d', ),
        weights=dict(
            argstr='-weights %s',
            sep=',',
        ),
        width_weighted=dict(
            argstr='-width_weighted',
            requires=('avgdim', ),
        ),
    )
    inputs = Average.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Average_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Average.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import SegStatsReconAll


def test_SegStatsReconAll_inputs():
    input_map = dict(
        annot=dict(
            argstr='--annot %s %s %s',
            mandatory=True,
            xor=('segmentation_file', 'annot', 'surf_label'),
        ),
        args=dict(argstr='%s', ),
        aseg=dict(extensions=None, ),
        avgwf_file=dict(argstr='--avgwfvol %s', ),
        avgwf_txt_file=dict(argstr='--avgwf %s', ),
        brain_vol=dict(argstr='--%s', ),
        brainmask_file=dict(
            argstr='--brainmask %s',
            extensions=None,
        ),
        calc_power=dict(argstr='--%s', ),
        calc_snr=dict(argstr='--snr', ),
        color_table_file=dict(
            argstr='--ctab %s',
            extensions=None,
            xor=('color_table_file', 'default_color_table', 'gca_color_table'),
        ),
        copy_inputs=dict(),
        cortex_vol_from_surf=dict(argstr='--surf-ctx-vol', ),
        default_color_table=dict(
            argstr='--ctab-default',
            xor=('color_table_file', 'default_color_table', 'gca_color_table'),
        ),
        empty=dict(argstr='--empty', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        etiv=dict(argstr='--etiv', ),
        etiv_only=dict(),
        euler=dict(argstr='--euler', ),
        exclude_ctx_gm_wm=dict(argstr='--excl-ctxgmwm', ),
        exclude_id=dict(argstr='--excludeid %d', ),
        frame=dict(argstr='--frame %d', ),
        gca_color_table=dict(
            argstr='--ctab-gca %s',
            extensions=None,
            xor=('color_table_file', 'default_color_table', 'gca_color_table'),
        ),
        in_file=dict(
            argstr='--i %s',
            extensions=None,
        ),
        in_intensity=dict(
            argstr='--in %s --in-intensity-name %s',
            extensions=None,
        ),
        intensity_units=dict(
            argstr='--in-intensity-units %s',
            requires=['in_intensity'],
        ),
        lh_orig_nofix=dict(
            extensions=None,
            mandatory=True,
        ),
        lh_pial=dict(
            extensions=None,
            mandatory=True,
        ),
        lh_white=dict(
            extensions=None,
            mandatory=True,
        ),
        mask_erode=dict(argstr='--maskerode %d', ),
        mask_file=dict(
            argstr='--mask %s',
            extensions=None,
        ),
        mask_frame=dict(requires=['mask_file'], ),
        mask_invert=dict(argstr='--maskinvert', ),
        mask_sign=dict(),
        mask_thresh=dict(argstr='--maskthresh %f', ),
        multiply=dict(argstr='--mul %f', ),
        non_empty_only=dict(argstr='--nonempty', ),
        partial_volume_file=dict(
            argstr='--pv %s',
            extensions=None,
        ),
        presurf_seg=dict(extensions=None, ),
        rh_orig_nofix=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_pial=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_white=dict(
            extensions=None,
            mandatory=True,
        ),
        ribbon=dict(
            extensions=None,
            mandatory=True,
        ),
        segment_id=dict(argstr='--id %s...', ),
        segmentation_file=dict(
            argstr='--seg %s',
            extensions=None,
            mandatory=True,
            xor=('segmentation_file', 'annot', 'surf_label'),
        ),
        sf_avg_file=dict(argstr='--sfavg %s', ),
        subcort_gm=dict(argstr='--subcortgray', ),
        subject_id=dict(
            argstr='--subject %s',
            mandatory=True,
            usedefault=True,
        ),
        subjects_dir=dict(),
        summary_file=dict(
            argstr='--sum %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        supratent=dict(argstr='--supratent', ),
        surf_label=dict(
            argstr='--slabel %s %s %s',
            mandatory=True,
            xor=('segmentation_file', 'annot', 'surf_label'),
        ),
        total_gray=dict(argstr='--totalgray', ),
        transform=dict(
            extensions=None,
            mandatory=True,
        ),
        vox=dict(argstr='--vox %s', ),
        wm_vol_from_surf=dict(argstr='--surf-wm-vol', ),
    )
    inputs = SegStatsReconAll.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SegStatsReconAll_outputs():
    output_map = dict(
        avgwf_file=dict(extensions=None, ),
        avgwf_txt_file=dict(extensions=None, ),
        sf_avg_file=dict(extensions=None, ),
        summary_file=dict(extensions=None, ),
    )
    outputs = SegStatsReconAll.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Code to build the documentation in the setup.py

To use this code, run::

    python setup.py build_sphinx
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open, str

# Standard library imports
import sys
import os
from os.path import join as pjoin
import zipfile
import warnings
import shutil
from distutils.cmd import Command
from distutils.command.clean import clean

_info_fname = pjoin(os.path.dirname(__file__), 'nipype', 'info.py')
INFO_VARS = {}
exec(str(open(_info_fname, 'rt').read()), {}, INFO_VARS)

DOC_BUILD_DIR = os.path.join('doc', '_build', 'html')
DOC_DOCTREES_DIR = os.path.join('doc', '_build', 'doctrees')

###############################################################################
# Distutils Command class for installing nipype to a temporary location.


class TempInstall(Command):
    temp_install_dir = os.path.join('build', 'install')

    def run(self):
        """ build and install nipype in a temporary location. """
        install = self.distribution.get_command_obj('install')
        install.install_scripts = self.temp_install_dir
        install.install_base = self.temp_install_dir
        install.install_platlib = self.temp_install_dir
        install.install_purelib = self.temp_install_dir
        install.install_data = self.temp_install_dir
        install.install_lib = self.temp_install_dir
        install.install_headers = self.temp_install_dir
        install.run()

        # Horrible trick to reload nipype with our temporary instal
        for key in list(sys.modules.keys()):
            if key.startswith('nipype'):
                sys.modules.pop(key, None)
        sys.path.append(os.path.abspath(self.temp_install_dir))
        # Pop the cwd
        sys.path.pop(0)
        import nipype

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass


###############################################################################
# Distutils Command class for API generation
class APIDocs(TempInstall):
    description = \
        """generate API docs """

    user_options = [
        ('None', None, 'this command has no options'),
    ]

    def run(self):
        # First build the project and install it to a temporary location.
        TempInstall.run(self)
        os.chdir('doc')
        try:
            # We are running the API-building script via an
            # system call, but overriding the import path.
            toolsdir = os.path.abspath(pjoin('..', 'tools'))
            for docbuilder in ['build_interface_docs.py']:
                build_templates = pjoin(toolsdir, docbuilder)
                cmd = """%s -c 'import sys; sys.path.append("%s"); sys.path.append("%s"); execfile("%s", dict(__name__="__main__"))'""" \
                    % (sys.executable,
                       toolsdir,
                       self.temp_install_dir,
                       build_templates)
                os.system(cmd)
        finally:
            os.chdir('..')


###############################################################################
# Code to copy the sphinx-generated html docs in the distribution.
def relative_path(filename):
    """ Return the relative path to the file, assuming the file is
        in the DOC_BUILD_DIR directory.
    """
    length = len(os.path.abspath(DOC_BUILD_DIR)) + 1
    return os.path.abspath(filename)[length:]


###############################################################################
# Distutils Command class build the docs
# Sphinx import.
try:
    from sphinx.setup_command import BuildDoc
except:
    MyBuildDoc = None
else:
    class MyBuildDoc(BuildDoc):
        """ Sub-class the standard sphinx documentation building system, to
            add logics for API generation and matplotlib's plot directive.
        """

        def run(self):
            self.run_command('api_docs')
            # We need to be in the doc directory for to plot_directive
            # and API generation to work
            """
            os.chdir('doc')
            try:
                BuildDoc.run(self)
            finally:
                os.chdir('..')
            """
            # It put's the build in a doc/doc/_build directory with the
            # above?!?!  I'm leaving the code above here but commented out
            # in case I'm missing something?
            BuildDoc.run(self)
            self.zip_docs()

        def zip_docs(self):
            if not os.path.exists(DOC_BUILD_DIR):
                raise OSError('Doc directory does not exist.')
            target_file = os.path.join('doc', 'documentation.zip')
            # ZIP_DEFLATED actually compresses the archive. However, there
            # will be a RuntimeError if zlib is not installed, so we check
            # for it. ZIP_STORED produces an uncompressed zip, but does not
            # require zlib.
            try:
                zf = zipfile.ZipFile(target_file, 'w',
                                     compression=zipfile.ZIP_DEFLATED)
            except RuntimeError:
                warnings.warn('zlib not installed, storing the docs '
                              'without compression')
                zf = zipfile.ZipFile(target_file, 'w',
                                     compression=zipfile.ZIP_STORED)

            for root, dirs, files in os.walk(DOC_BUILD_DIR):
                relative = relative_path(root)
                if not relative.startswith('.doctrees'):
                    for f in files:
                        zf.write(os.path.join(root, f),
                                 os.path.join(relative, 'html_docs', f))
            zf.close()

        def finalize_options(self):
            """ Override the default for the documentation build
                directory.
            """
            self.build_dir = os.path.join(*DOC_BUILD_DIR.split(os.sep)[:-1])
            BuildDoc.finalize_options(self)

###############################################################################
# Distutils Command class to clean


class Clean(clean):

    def run(self):
        clean.run(self)
        api_path = os.path.join('doc', 'api', 'generated')
        if os.path.exists(api_path):
            print("Removing %s" % api_path)
            shutil.rmtree(api_path)
        interface_path = os.path.join('doc', 'interfaces', 'generated')
        if os.path.exists(interface_path):
            print("Removing %s" % interface_path)
            shutil.rmtree(interface_path)
        if os.path.exists(DOC_BUILD_DIR):
            print("Removing %s" % DOC_BUILD_DIR)
            shutil.rmtree(DOC_BUILD_DIR)
        if os.path.exists(DOC_DOCTREES_DIR):
            print("Removing %s" % DOC_DOCTREES_DIR)
            shutil.rmtree(DOC_DOCTREES_DIR)


# The command classes for distutils, used by the setup.py
cmdclass = {'build_sphinx': MyBuildDoc,
            'api_docs': APIDocs,
            'clean': Clean,
            }



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Tests for workflow callbacks
"""

from builtins import object

from time import sleep
import pytest
import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe


def func():
    return


def bad_func():
    raise Exception


class Status(object):
    def __init__(self):
        self.statuses = []

    def callback(self, node, status, result=None):
        self.statuses.append((node.name, status))


@pytest.mark.parametrize("plugin", ['Linear', 'MultiProc', 'LegacyMultiProc'])
def test_callback_normal(tmpdir, plugin):
    tmpdir.chdir()

    so = Status()
    wf = pe.Workflow(name='test', base_dir=tmpdir.strpath)
    f_node = pe.Node(
        niu.Function(function=func, input_names=[], output_names=[]),
        name='f_node')
    wf.add_nodes([f_node])
    wf.config['execution'] = {
        'crashdump_dir': wf.base_dir,
        'poll_sleep_duration': 2
    }
    wf.run(plugin=plugin, plugin_args={'status_callback': so.callback})
    assert so.statuses == [('f_node', 'start'), ('f_node', 'end')]


@pytest.mark.parametrize("plugin", ['Linear', 'MultiProc', 'LegacyMultiProc'])
@pytest.mark.parametrize("stop_on_first_crash", [False, True])
def test_callback_exception(tmpdir, plugin, stop_on_first_crash):
    tmpdir.chdir()

    so = Status()
    wf = pe.Workflow(name='test', base_dir=tmpdir.strpath)
    f_node = pe.Node(
        niu.Function(function=bad_func, input_names=[], output_names=[]),
        name='f_node')
    wf.add_nodes([f_node])
    wf.config['execution'] = {
        'crashdump_dir': wf.base_dir,
        'stop_on_first_crash': stop_on_first_crash,
        'poll_sleep_duration': 2
    }
    with pytest.raises(Exception):
        wf.run(plugin=plugin, plugin_args={'status_callback': so.callback})

    sleep(0.5)  # Wait for callback to be called (python 2.7)
    assert so.statuses == [('f_node', 'start'), ('f_node', 'exception')]



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tracks import StreamlineTractography


def test_StreamlineTractography_inputs():
    input_map = dict(
        gfa_thresh=dict(
            mandatory=True,
            usedefault=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        in_model=dict(extensions=None, ),
        in_peaks=dict(extensions=None, ),
        min_angle=dict(
            mandatory=True,
            usedefault=True,
        ),
        multiprocess=dict(
            mandatory=True,
            usedefault=True,
        ),
        num_seeds=dict(
            mandatory=True,
            usedefault=True,
        ),
        out_prefix=dict(),
        peak_threshold=dict(
            mandatory=True,
            usedefault=True,
        ),
        save_seeds=dict(
            mandatory=True,
            usedefault=True,
        ),
        seed_coord=dict(extensions=None, ),
        seed_mask=dict(extensions=None, ),
        tracking_mask=dict(extensions=None, ),
    )
    inputs = StreamlineTractography.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_StreamlineTractography_outputs():
    output_map = dict(
        gfa=dict(extensions=None, ),
        odf_peaks=dict(extensions=None, ),
        out_seeds=dict(extensions=None, ),
        tracks=dict(extensions=None, ),
    )
    outputs = StreamlineTractography.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Dot


def test_Dot_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        demean=dict(argstr='-demean', ),
        docoef=dict(argstr='-docoef', ),
        docor=dict(argstr='-docor', ),
        dodice=dict(argstr='-dodice', ),
        dodot=dict(argstr='-dodot', ),
        doeta2=dict(argstr='-doeta2', ),
        dosums=dict(argstr='-dosums', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        full=dict(argstr='-full', ),
        in_files=dict(
            argstr='%s ...',
            position=-2,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        mrange=dict(argstr='-mrange %s %s', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr=' |& tee %s',
            extensions=None,
            position=-1,
        ),
        outputtype=dict(),
        show_labels=dict(argstr='-show_labels', ),
        upper=dict(argstr='-upper', ),
    )
    inputs = Dot.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Dot_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Dot.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
:mod:`nipype.sphinxext.plot_workflow` -- Workflow plotting extension
====================================================================


A directive for including a nipype workflow graph in a Sphinx document.

This code is forked from the plot_figure sphinx extension of matplotlib.

By default, in HTML output, `workflow` will include a .png file with a
link to a high-res .png.  In LaTeX output, it will include a
.pdf.
The source code for the workflow may be included as **inline content** to
the directive `workflow`::

  .. workflow ::
      :graph2use: flat
      :simple_form: no

      from nipype.workflows.dmri.camino.connectivity_mapping import create_connectivity_pipeline
      wf = create_connectivity_pipeline()


For example, the following graph has been generated inserting the previous
code block in this documentation:

.. workflow ::
  :graph2use: flat
  :simple_form: no

  from nipype.workflows.dmri.camino.connectivity_mapping import create_connectivity_pipeline
  wf = create_connectivity_pipeline()


Options
-------

The ``workflow`` directive supports the following options:
    graph2use : {'hierarchical', 'colored', 'flat', 'orig', 'exec'}
        Specify the type of graph to be generated.
    simple_form: bool
        Whether the graph will be in detailed or simple form.
    format : {'python', 'doctest'}
        Specify the format of the input
    include-source : bool
        Whether to display the source code. The default can be changed
        using the `workflow_include_source` variable in conf.py
    encoding : str
        If this source file is in a non-UTF8 or non-ASCII encoding,
        the encoding must be specified using the `:encoding:` option.
        The encoding will not be inferred using the ``-*- coding -*-``
        metacomment.

Additionally, this directive supports all of the options of the
`image` directive, except for `target` (since workflow will add its own
target).  These include `alt`, `height`, `width`, `scale`, `align` and
`class`.

Configuration options
---------------------

The workflow directive has the following configuration options:
    graph2use
        Select a graph type to use
    simple_form
        determines if the node name shown in the visualization is either of the form nodename
        (package) when set to True or nodename.Class.package when set to False.
    wf_include_source
        Default value for the include-source option
    wf_html_show_source_link
        Whether to show a link to the source in HTML.
    wf_pre_code
        Code that should be executed before each workflow.
    wf_basedir
        Base directory, to which ``workflow::`` file names are relative
        to.  (If None or empty, file names are relative to the
        directory where the file containing the directive is.)
    wf_formats
        File formats to generate. List of tuples or strings::
            [(suffix, dpi), suffix, ...]
        that determine the file format and the DPI. For entries whose
        DPI was omitted, sensible defaults are chosen. When passing from
        the command line through sphinx_build the list should be passed as
        suffix:dpi,suffix:dpi, ....
    wf_html_show_formats
        Whether to show links to the files in HTML.
    wf_rcparams
        A dictionary containing any non-standard rcParams that should
        be applied before each workflow.
    wf_apply_rcparams
        By default, rcParams are applied when `context` option is not used in
        a workflow directive.  This configuration option overrides this behavior
        and applies rcParams before each workflow.
    wf_working_directory
        By default, the working directory will be changed to the directory of
        the example, so the code can get at its data files, if any.  Also its
        path will be added to `sys.path` so it can import any helper modules
        sitting beside it.  This configuration option can be used to specify
        a central directory (also added to `sys.path`) where data files and
        helper modules for all code are located.
    wf_template
        Provide a customized template for preparing restructured text.

"""
from __future__ import print_function, division, absolute_import, unicode_literals

import sys
import os
import shutil
import io
import re
import textwrap
from os.path import relpath
from errno import EEXIST
import traceback

missing_imports = []
try:
    from docutils.parsers.rst import directives
    from docutils.parsers.rst.directives.images import Image
    align = Image.align
except ImportError as e:
    missing_imports = [str(e)]

try:
    # Sphinx depends on either Jinja or Jinja2
    import jinja2

    def format_template(template, **kw):
        return jinja2.Template(template).render(**kw)
except ImportError as e:
    missing_imports.append(str(e))
    try:
        import jinja

        def format_template(template, **kw):
            return jinja.from_string(template, **kw)

        missing_imports.pop()
    except ImportError as e:
        missing_imports.append(str(e))

from builtins import str, bytes

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


def _mkdirp(folder):
    """
    Equivalent to bash's mkdir -p
    """
    if sys.version_info > (3, 4, 1):
        os.makedirs(folder, exist_ok=True)
        return folder

    try:
        os.makedirs(folder)
    except OSError as exc:
        if exc.errno != EEXIST or not os.path.isdir(folder):
            raise

    return folder


def wf_directive(name, arguments, options, content, lineno, content_offset,
                 block_text, state, state_machine):
    if len(missing_imports) == 0:
        return run(arguments, content, options, state_machine, state, lineno)
    else:
        raise ImportError('\n'.join(missing_imports))


wf_directive.__doc__ = __doc__


def _option_boolean(arg):
    if not arg or not arg.strip():
        # no argument given, assume used as a flag
        return True
    elif arg.strip().lower() in ('no', '0', 'false'):
        return False
    elif arg.strip().lower() in ('yes', '1', 'true'):
        return True
    else:
        raise ValueError('"%s" unknown boolean' % arg)


def _option_graph2use(arg):
    return directives.choice(
        arg, ('hierarchical', 'colored', 'flat', 'orig', 'exec'))


def _option_context(arg):
    if arg in [None, 'reset', 'close-figs']:
        return arg
    raise ValueError("argument should be None or 'reset' or 'close-figs'")


def _option_format(arg):
    return directives.choice(arg, ('python', 'doctest'))


def _option_align(arg):
    return directives.choice(
        arg, ("top", "middle", "bottom", "left", "center", "right"))


def mark_wf_labels(app, document):
    """
    To make graphs referenceable, we need to move the reference from
    the "htmlonly" (or "latexonly") node to the actual figure node
    itself.
    """
    for name, explicit in list(document.nametypes.items()):
        if not explicit:
            continue
        labelid = document.nameids[name]
        if labelid is None:
            continue
        node = document.ids[labelid]
        if node.tagname in ('html_only', 'latex_only'):
            for n in node:
                if n.tagname == 'figure':
                    sectname = name
                    for c in n:
                        if c.tagname == 'caption':
                            sectname = c.astext()
                            break

                    node['ids'].remove(labelid)
                    node['names'].remove(name)
                    n['ids'].append(labelid)
                    n['names'].append(name)
                    document.settings.env.labels[name] = \
                        document.settings.env.docname, labelid, sectname
                    break


def setup(app):
    setup.app = app
    setup.config = app.config
    setup.confdir = app.confdir

    options = {
        'alt': directives.unchanged,
        'height': directives.length_or_unitless,
        'width': directives.length_or_percentage_or_unitless,
        'scale': directives.nonnegative_int,
        'align': _option_align,
        'class': directives.class_option,
        'include-source': _option_boolean,
        'format': _option_format,
        'context': _option_context,
        'nofigs': directives.flag,
        'encoding': directives.encoding,
        'graph2use': _option_graph2use,
        'simple_form': _option_boolean
    }

    app.add_directive('workflow', wf_directive, True, (0, 2, False), **options)
    app.add_config_value('graph2use', 'hierarchical', 'html')
    app.add_config_value('simple_form', True, 'html')
    app.add_config_value('wf_pre_code', None, True)
    app.add_config_value('wf_include_source', False, True)
    app.add_config_value('wf_html_show_source_link', True, True)
    app.add_config_value('wf_formats', ['png', 'svg', 'pdf'], True)
    app.add_config_value('wf_basedir', None, True)
    app.add_config_value('wf_html_show_formats', True, True)
    app.add_config_value('wf_rcparams', {}, True)
    app.add_config_value('wf_apply_rcparams', False, True)
    app.add_config_value('wf_working_directory', None, True)
    app.add_config_value('wf_template', None, True)

    app.connect('doctree-read'.encode()
                if PY2 else 'doctree-read', mark_wf_labels)

    metadata = {'parallel_read_safe': True, 'parallel_write_safe': True}
    return metadata


# ------------------------------------------------------------------------------
# Doctest handling
# ------------------------------------------------------------------------------


def contains_doctest(text):
    try:
        # check if it's valid Python as-is
        compile(text, '<string>', 'exec')
        return False
    except SyntaxError:
        pass
    r = re.compile(r'^\s*>>>', re.M)
    m = r.search(text)
    return bool(m)


def unescape_doctest(text):
    """
    Extract code from a piece of text, which contains either Python code
    or doctests.
    """
    if not contains_doctest(text):
        return text

    code = ""
    for line in text.split("\n"):
        m = re.match(r'^\s*(>>>|\.\.\.) (.*)$', line)
        if m:
            code += m.group(2) + "\n"
        elif line.strip():
            code += "# " + line.strip() + "\n"
        else:
            code += "\n"
    return code


def remove_coding(text):
    """
    Remove the coding comment, which exec doesn't like.
    """
    sub_re = re.compile("^#\s*-\*-\s*coding:\s*.*-\*-$", flags=re.MULTILINE)
    return sub_re.sub("", text)


# ------------------------------------------------------------------------------
# Template
# ------------------------------------------------------------------------------

TEMPLATE = """
{{ source_code }}
{{ only_html }}
   {% for img in images %}
   .. figure:: {{ build_dir }}/{{ img.basename }}.{{ default_fmt }}
      {% for option in options -%}
      {{ option }}
      {% endfor %}
      {% if html_show_formats and multi_image -%}
        (
        {%- for fmt in img.formats -%}
        {%- if not loop.first -%}, {% endif -%}
        `{{ fmt }} <{{ dest_dir }}/{{ img.basename }}.{{ fmt }}>`__
        {%- endfor -%}
        )
      {%- endif -%}
      {{ caption }}
   {% endfor %}
   {% if source_link or (html_show_formats and not multi_image) %}
   (
   {%- if source_link -%}
   `Source code <{{ source_link }}>`__
   {%- endif -%}
   {%- if html_show_formats and not multi_image -%}
     {%- for img in images -%}
       {%- for fmt in img.formats -%}
         {%- if source_link or not loop.first -%}, {% endif -%}
         `{{ fmt }} <{{ dest_dir }}/{{ img.basename }}.{{ fmt }}>`__
       {%- endfor -%}
     {%- endfor -%}
   {%- endif -%}
   )
   {% endif %}
{{ only_latex }}
   {% for img in images %}
   {% if 'pdf' in img.formats -%}
   .. figure:: {{ build_dir }}/{{ img.basename }}.pdf
      {% for option in options -%}
      {{ option }}
      {% endfor %}
      {{ caption }}
   {% endif -%}
   {% endfor %}
{{ only_texinfo }}
   {% for img in images %}
   .. image:: {{ build_dir }}/{{ img.basename }}.png
      {% for option in options -%}
      {{ option }}
      {% endfor %}
   {% endfor %}
"""

exception_template = """
.. htmlonly::
   [`source code <%(linkdir)s/%(basename)s.py>`__]
Exception occurred rendering plot.
"""

# the context of the plot for all directives specified with the
# :context: option
wf_context = dict()


class ImageFile(object):
    def __init__(self, basename, dirname):
        self.basename = basename
        self.dirname = dirname
        self.formats = []

    def filename(self, fmt):
        return os.path.join(self.dirname, "%s.%s" % (self.basename, fmt))

    def filenames(self):
        return [self.filename(fmt) for fmt in self.formats]


def out_of_date(original, derived):
    """
    Returns True if derivative is out-of-date wrt original,
    both of which are full file paths.
    """
    return (not os.path.exists(derived)
            or (os.path.exists(original)
                and os.stat(derived).st_mtime < os.stat(original).st_mtime))


class GraphError(RuntimeError):
    pass


def run_code(code, code_path, ns=None, function_name=None):
    """
    Import a Python module from a path, and run the function given by
    name, if function_name is not None.
    """

    # Change the working directory to the directory of the example, so
    # it can get at its data files, if any.  Add its path to sys.path
    # so it can import any helper modules sitting beside it.
    pwd = str(os.getcwd())
    old_sys_path = list(sys.path)
    if setup.config.wf_working_directory is not None:
        try:
            os.chdir(setup.config.wf_working_directory)
        except OSError as err:
            raise OSError(
                str(err) + '\n`wf_working_directory` option in'
                'Sphinx configuration file must be a valid '
                'directory path')
        except TypeError as err:
            raise TypeError(
                str(err) + '\n`wf_working_directory` option in '
                'Sphinx configuration file must be a string or '
                'None')
        sys.path.insert(0, setup.config.wf_working_directory)
    elif code_path is not None:
        dirname = os.path.abspath(os.path.dirname(code_path))
        os.chdir(dirname)
        sys.path.insert(0, dirname)

    # Reset sys.argv
    old_sys_argv = sys.argv
    sys.argv = [code_path]

    # Redirect stdout
    stdout = sys.stdout
    if PY3:
        sys.stdout = io.StringIO()
    else:
        from cStringIO import StringIO
        sys.stdout = StringIO()

    # Assign a do-nothing print function to the namespace.  There
    # doesn't seem to be any other way to provide a way to (not) print
    # that works correctly across Python 2 and 3.
    def _dummy_print(*arg, **kwarg):
        pass

    try:
        try:
            code = unescape_doctest(code)
            if ns is None:
                ns = {}
            if not ns:
                if setup.config.wf_pre_code is not None:
                    exec(str(setup.config.wf_pre_code), ns)
            ns['print'] = _dummy_print
            if "__main__" in code:
                exec("__name__ = '__main__'", ns)
            code = remove_coding(code)
            exec(code, ns)
            if function_name is not None:
                exec(function_name + "()", ns)
        except (Exception, SystemExit) as err:
            raise GraphError(traceback.format_exc())
    finally:
        os.chdir(pwd)
        sys.argv = old_sys_argv
        sys.path[:] = old_sys_path
        sys.stdout = stdout
    return ns


def get_wf_formats(config):
    default_dpi = {'png': 80, 'hires.png': 200, 'pdf': 200}
    formats = []
    wf_formats = config.wf_formats
    if isinstance(wf_formats, (str, bytes)):
        # String Sphinx < 1.3, Split on , to mimic
        # Sphinx 1.3 and later. Sphinx 1.3 always
        # returns a list.
        wf_formats = wf_formats.split(',')
    for fmt in wf_formats:
        if isinstance(fmt, (str, bytes)):
            if ':' in fmt:
                suffix, dpi = fmt.split(':')
                formats.append((str(suffix), int(dpi)))
            else:
                formats.append((fmt, default_dpi.get(fmt, 80)))
        elif isinstance(fmt, (tuple, list)) and len(fmt) == 2:
            formats.append((str(fmt[0]), int(fmt[1])))
        else:
            raise GraphError('invalid image format "%r" in wf_formats' % fmt)
    return formats


def render_figures(code,
                   code_path,
                   output_dir,
                   output_base,
                   context,
                   function_name,
                   config,
                   graph2use,
                   simple_form,
                   context_reset=False,
                   close_figs=False):
    """
    Run a nipype workflow creation script and save the graph in *output_dir*.
    Save the images under *output_dir* with file names derived from
    *output_base*
    """
    formats = get_wf_formats(config)
    ns = wf_context if context else {}
    if context_reset:
        wf_context.clear()

    run_code(code, code_path, ns, function_name)
    img = ImageFile(output_base, output_dir)

    for fmt, dpi in formats:
        try:
            img_path = img.filename(fmt)
            imgname, ext = os.path.splitext(os.path.basename(img_path))
            ns['wf'].base_dir = output_dir
            src = ns['wf'].write_graph(
                imgname,
                format=ext[1:],
                graph2use=graph2use,
                simple_form=simple_form)
            shutil.move(src, img_path)
        except Exception:
            raise GraphError(traceback.format_exc())

        img.formats.append(fmt)

    return [(code, [img])]


def run(arguments, content, options, state_machine, state, lineno):
    document = state_machine.document
    config = document.settings.env.config
    nofigs = 'nofigs' in options

    formats = get_wf_formats(config)
    default_fmt = formats[0][0]

    graph2use = options.get('graph2use', 'hierarchical')
    simple_form = options.get('simple_form', True)

    options.setdefault('include-source', config.wf_include_source)
    keep_context = 'context' in options
    context_opt = None if not keep_context else options['context']

    rst_file = document.attributes['source']
    rst_dir = os.path.dirname(rst_file)

    if len(arguments):
        if not config.wf_basedir:
            source_file_name = os.path.join(setup.app.builder.srcdir,
                                            directives.uri(arguments[0]))
        else:
            source_file_name = os.path.join(setup.confdir, config.wf_basedir,
                                            directives.uri(arguments[0]))

        # If there is content, it will be passed as a caption.
        caption = '\n'.join(content)

        # If the optional function name is provided, use it
        if len(arguments) == 2:
            function_name = arguments[1]
        else:
            function_name = None

        with io.open(source_file_name, 'r', encoding='utf-8') as fd:
            code = fd.read()
        output_base = os.path.basename(source_file_name)
    else:
        source_file_name = rst_file
        code = textwrap.dedent("\n".join([str(c) for c in content]))
        counter = document.attributes.get('_wf_counter', 0) + 1
        document.attributes['_wf_counter'] = counter
        base, _ = os.path.splitext(os.path.basename(source_file_name))
        output_base = '%s-%d.py' % (base, counter)
        function_name = None
        caption = ''

    base, source_ext = os.path.splitext(output_base)
    if source_ext in ('.py', '.rst', '.txt'):
        output_base = base
    else:
        source_ext = ''

    # ensure that LaTeX includegraphics doesn't choke in foo.bar.pdf filenames
    output_base = output_base.replace('.', '-')

    # is it in doctest format?
    is_doctest = contains_doctest(code)
    if 'format' in options:
        if options['format'] == 'python':
            is_doctest = False
        else:
            is_doctest = True

    # determine output directory name fragment
    source_rel_name = relpath(source_file_name, setup.confdir)
    source_rel_dir = os.path.dirname(source_rel_name)
    while source_rel_dir.startswith(os.path.sep):
        source_rel_dir = source_rel_dir[1:]

    # build_dir: where to place output files (temporarily)
    build_dir = os.path.join(
        os.path.dirname(setup.app.doctreedir), 'wf_directive', source_rel_dir)
    # get rid of .. in paths, also changes pathsep
    # see note in Python docs for warning about symbolic links on Windows.
    # need to compare source and dest paths at end
    build_dir = os.path.normpath(build_dir)

    if not os.path.exists(build_dir):
        os.makedirs(build_dir)

    # output_dir: final location in the builder's directory
    dest_dir = os.path.abspath(
        os.path.join(setup.app.builder.outdir, source_rel_dir))
    if not os.path.exists(dest_dir):
        os.makedirs(dest_dir)  # no problem here for me, but just use built-ins

    # how to link to files from the RST file
    dest_dir_link = os.path.join(
        relpath(setup.confdir, rst_dir), source_rel_dir).replace(
            os.path.sep, '/')
    try:
        build_dir_link = relpath(build_dir, rst_dir).replace(os.path.sep, '/')
    except ValueError:
        # on Windows, relpath raises ValueError when path and start are on
        # different mounts/drives
        build_dir_link = build_dir
    source_link = dest_dir_link + '/' + output_base + source_ext

    # make figures
    try:
        results = render_figures(
            code,
            source_file_name,
            build_dir,
            output_base,
            keep_context,
            function_name,
            config,
            graph2use,
            simple_form,
            context_reset=context_opt == 'reset',
            close_figs=context_opt == 'close-figs')
        errors = []
    except GraphError as err:
        reporter = state.memo.reporter
        sm = reporter.system_message(
            2,
            "Exception occurred in plotting %s\n from %s:\n%s" %
            (output_base, source_file_name, err),
            line=lineno)
        results = [(code, [])]
        errors = [sm]

    # Properly indent the caption
    caption = '\n'.join(
        '      ' + line.strip() for line in caption.split('\n'))

    # generate output restructuredtext
    total_lines = []
    for j, (code_piece, images) in enumerate(results):
        if options['include-source']:
            if is_doctest:
                lines = ['']
                lines += [row.rstrip() for row in code_piece.split('\n')]
            else:
                lines = ['.. code-block:: python', '']
                lines += [
                    '    %s' % row.rstrip() for row in code_piece.split('\n')
                ]
            source_code = "\n".join(lines)
        else:
            source_code = ""

        if nofigs:
            images = []

        opts = [
            ':%s: %s' % (key, val) for key, val in list(options.items())
            if key in ('alt', 'height', 'width', 'scale', 'align', 'class')
        ]

        only_html = ".. only:: html"
        only_latex = ".. only:: latex"
        only_texinfo = ".. only:: texinfo"

        # Not-None src_link signals the need for a source link in the generated
        # html
        if j == 0 and config.wf_html_show_source_link:
            src_link = source_link
        else:
            src_link = None

        result = format_template(
            config.wf_template or TEMPLATE,
            default_fmt=default_fmt,
            dest_dir=dest_dir_link,
            build_dir=build_dir_link,
            source_link=src_link,
            multi_image=len(images) > 1,
            only_html=only_html,
            only_latex=only_latex,
            only_texinfo=only_texinfo,
            options=opts,
            images=images,
            source_code=source_code,
            html_show_formats=config.wf_html_show_formats and len(images),
            caption=caption)

        total_lines.extend(result.split("\n"))
        total_lines.extend("\n")

    if total_lines:
        state_machine.insert_input(total_lines, source=source_file_name)

    # copy image files to builder's output directory, if necessary
    _mkdirp(dest_dir)
    for code_piece, images in results:
        for img in images:
            for fn in img.filenames():
                destimg = os.path.join(dest_dir, os.path.basename(fn))
                if fn != destimg:
                    shutil.copyfile(fn, destimg)

    # copy script (if necessary)
    target_name = os.path.join(dest_dir, output_base + source_ext)
    with io.open(target_name, 'w', encoding="utf-8") as f:
        if source_file_name == rst_file:
            code_escaped = unescape_doctest(code)
        else:
            code_escaped = code
        f.write(code_escaped)

    return errors



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class IntensityDifferenceMetricInputSpec(CommandLineInputSpec):
    sensitivityThreshold = traits.Float(
        desc=
        "This parameter should be between 0 and 1, and defines how sensitive the metric should be to the intensity changes.",
        argstr="--sensitivityThreshold %f")
    changingBandSize = traits.Int(
        desc=
        "How far (in mm) from the boundary of the segmentation should the intensity changes be considered.",
        argstr="--changingBandSize %d")
    baselineVolume = File(
        position=-4,
        desc="Baseline volume to be compared to",
        exists=True,
        argstr="%s")
    baselineSegmentationVolume = File(
        position=-3,
        desc=
        "Label volume that contains segmentation of the structure of interest in the baseline volume.",
        exists=True,
        argstr="%s")
    followupVolume = File(
        position=-2,
        desc="Followup volume to be compare to the baseline",
        exists=True,
        argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output volume to keep the results of change quantification.",
        argstr="%s")
    reportFileName = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Report file name",
        argstr="--reportFileName %s")


class IntensityDifferenceMetricOutputSpec(TraitedSpec):
    outputVolume = File(
        position=-1,
        desc="Output volume to keep the results of change quantification.",
        exists=True)
    reportFileName = File(desc="Report file name", exists=True)


class IntensityDifferenceMetric(SEMLikeCommandLine):
    """title:
  Intensity Difference Change Detection (FAST)


category:
  Quantification.ChangeQuantification


description:
  Quantifies the changes between two spatially aligned images based on the pixel-wise difference of image intensities.


version: 0.1

contributor: Andrey Fedorov

acknowledgements:


"""

    input_spec = IntensityDifferenceMetricInputSpec
    output_spec = IntensityDifferenceMetricOutputSpec
    _cmd = "IntensityDifferenceMetric "
    _outputs_filenames = {
        'outputVolume': 'outputVolume.nii',
        'reportFileName': 'reportFileName'
    }



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
from distutils.version import LooseVersion

from .info import (LONG_DESCRIPTION as __doc__, URL as __url__, STATUS as
                   __status__, __version__)
from .utils.config import NipypeConfig
from .utils.logger import Logging
from .refs import due
from .pkg_info import get_pkg_info as _get_pkg_info

try:
    import faulthandler
    faulthandler.enable()
except (ImportError, IOError) as e:
    pass

config = NipypeConfig()
logging = Logging(config)


class NipypeTester(object):
    def __call__(self, doctests=True, parallel=False):
        try:
            import pytest
        except ImportError:
            raise RuntimeError(
                'py.test not installed, run: pip install pytest')
        args = []
        if not doctests:
            args.extend(['-p', 'no:doctest'])
        if parallel:
            try:
                import xdist
            except ImportError:
                raise RuntimeError(
                    "pytest-xdist required for parallel run")
            args.append('-n auto')
        args.append(os.path.dirname(__file__))
        pytest.main(args=args)


test = NipypeTester()


def get_info():
    """Returns package information"""
    return _get_pkg_info(os.path.dirname(__file__))


from .pipeline import Node, MapNode, JoinNode, Workflow
from .interfaces import (DataGrabber, DataSink, SelectFiles, IdentityInterface,
                         Rename, Function, Select, Merge)


def check_latest_version(raise_exception=False):
    """Check for the latest version of the library

    parameters:
    raise_exception: boolean
        Raise a RuntimeError if a bad version is being used
    """
    import etelemetry
    logger = logging.getLogger('nipype.utils')

    INIT_MSG = "Running {packname} version {version} (latest: {latest})".format

    latest = {"version": 'Unknown', "bad_versions": []}
    result = None
    try:
        result = etelemetry.get_project("nipy/nipype")
    except Exception as e:
        logger.warning("Could not check for version updates: \n%s", e)
    finally:
        if result:
            latest.update(**result)
            if LooseVersion(__version__) != LooseVersion(latest["version"]):
                logger.info(INIT_MSG(packname='nipype',
                                     version=__version__,
                                     latest=latest["version"]))
            if latest["bad_versions"] and \
                    any([LooseVersion(__version__) == LooseVersion(ver)
                         for ver in latest["bad_versions"]]):
                message = ('You are using a version of Nipype with a critical '
                           'bug. Please use a different version.')
                if raise_exception:
                    raise RuntimeError(message)
                else:
                    logger.critical(message)
    return latest

# Run telemetry on import for interactive sessions, such as IPython, Jupyter notebooks, Python REPL
if config.getboolean('execution', 'check_version'):
    import __main__
    if not hasattr(__main__, '__file__'):
        from .interfaces.base import BaseInterface
        if BaseInterface._etelemetry_version_data is None:
            BaseInterface._etelemetry_version_data = check_latest_version()



from nipype.interfaces.base import traits
from nipype.interfaces.base import TraitedSpec
from nipype.interfaces.matlab import MatlabCommand, MatlabInputSpec


class HelloWorldInputSpec(MatlabInputSpec):
    name = traits.Str(mandatory=True,
                      desc='Name of person to say hello to')


class HelloWorldOutputSpec(TraitedSpec):
    matlab_output = traits.Str()


class HelloWorld(MatlabCommand):
    """Basic Hello World that displays Hello <name> in MATLAB

    Returns
    -------

    matlab_output : capture of matlab output which may be
                    parsed by user to get computation results

    Examples
    --------

    >>> hello = HelloWorld()
    >>> hello.inputs.name = 'hello_world'
    >>> out = hello.run()
    >>> print out.outputs.matlab_output
    """
    input_spec = HelloWorldInputSpec
    output_spec = HelloWorldOutputSpec

    def _my_script(self):
        """This is where you implement your script"""
        script = """
        disp('Hello %s Python')
        two = 1 + 1
        """ % (self.inputs.name)
        return script

    def run(self, **inputs):
        # Inject your script
        self.inputs.script = self._my_script()
        results = super(MatlabCommand, self).run(**inputs)
        stdout = results.runtime.stdout
        # Attach stdout to outputs to access matlab results
        results.outputs.matlab_output = stdout
        return results

    def _list_outputs(self):
        outputs = self._outputs().get()
        return outputs
